$(document).on 'ready page:load turbolinks:load', ->
  $('table.editable_rows').each ->
    initEditableRows(this)
    initCreation(this)

initEditableRows = (table) ->
  $table = $(table)
  $table.find('a.edit').click (event) ->
    event.preventDefault()
    initRowEditing($(this))
  $table.find('a.destroy').click (event) ->
    event.preventDefault()
    event.stopPropagation()
    $button = $(this)
    confirmation = $button.data('confirm')
    if !confirmation or confirm(confirmation)
      $row = $button.closest('tr')
      $.ajax
        type: $button.data('method')
        url: $button.attr('href') + '.json'
        success: () ->
          $row.remove()
          dispatchChangeEvent($table, 'delete')
        error: (data) -> setErrors($row, data)

  initRowEditing = ($editButton) ->
    $controlsCell = $editButton.parent('td')
    $row = $controlsCell.parent('tr')
    orygControlHtml = $controlsCell.find('a').detach()
    initControls($controlsCell)
    initInputs($row)

    $row.find('td[data] input, td[data-name] input, td[data-name] select').keypress (event) ->
      if event.which == 13
        event.preventDefault()
        save()

    $controlsCell.find('.save').click (event) ->
      event.preventDefault()
      save()

    save = () ->
      clearErrors($row)
      data = {id: $row.data('id')}
      $row.find('td[data], td[data-name]').each ->
        $cell = $(this)
        name = if $cell.attr('data') then 'data' else 'data-name'
        if $cell.data('image')
          $form = $cell.find('form')
          $.ajax
            type: $form.attr('method')
            url: $form.attr('action')
            data: new FormData($form[0])
            processData: false
            contentType: false
            success: (data, textStatus, jqXHR) ->
              $cell.html(data)
        else
          data[$cell.attr(name)] = $cell.find('input, select').val()

      ajaxData = {}
      ajaxData[$table.data('resource')] = data
      $.ajax
        type: "PUT"
        url: window.location.href + "/#{$row.data('id')}.json"
        data: ajaxData
        success: () ->
          restoreRow(true)
          dispatchChangeEvent($table, 'update')
        error: (data) -> setErrors($row, data)

    restoreRow = (updated) ->
      $controlsCell.html(orygControlHtml)
      $row.find('td[data], td[data-name]').each ->
        $cell = $(this)
        $input = $cell.find('input, select')
        if updated
          if $cell.find('form').length == 0
            if $input.prop('tagName') == 'SELECT'
              $cell.html($input.find("option[value=#{$input.val()}]").html())
            else
              $cell.html($input.val())
        else
          $cell.html($cell.data('undo'))

    $controlsCell.find('.cancel').click (event) ->
      event.preventDefault()
      restoreRow(false)

initCreation = (table) ->
  $table = $(table)
  $createButton = $table.find('a.create')
  $row = $createButton.closest('tr')
  initInputs($row)

  $row.find('td[data], td[data-name]').keypress (event) ->
    if event.which == 13
      event.preventDefault()
      create()

  $createButton.click (event) ->
    event.preventDefault()
    create()

  create = () ->
    clearErrors($row)
    data = {}
    $row.find('td[data]').each ->
      $cell = $(this)
      data[$cell.attr('data')] = $cell.find('input, select').val()
    $row.find('td[data-name]').each ->
      $cell = $(this)
      data[$cell.attr('data-name')] = $cell.find('input, select').val()
    ajaxData = {}
    ajaxData[$table.data('resource')] = data
    $.ajax
      type: "POST"
      url: window.location.href
      data: ajaxData
      success: (data) ->
        newTableBody = $("<div>#{data}</div>").find('table.editable_rows tbody').html()
        $table.find('tbody').html(newTableBody)
        $row.find('input:text').val('')
        initEditableRows(table)
        dispatchChangeEvent($table, 'create')
      error: (data) -> setErrors($row, data)

initInputs = ($row) ->
  $row.find('td[data], td[data-name]').each ->
    $cell = $(this)
    $cell.data('undo', $cell.html())
    if $cell.data('select')
      value = $cell.html()
      $cell.html("<select>#{$cell.data('select')}</select><span class='has-error'/>")
      $select = $cell.find('select')
      $select.val($select.find("option:contains(#{value})").val())
    else if $cell.data('image')
      $cell.html("#{$cell.data('image')}</input><span class='has-error'/>")
    else
      value = $cell.html()
      $cell.html("<input value='#{value}'><span class='has-error'/>")

initControls = ($cell) ->
  $cell.html("<a class='btn btn-primary btn-xs save' href='#'><i class='fa fa-check'/></a> <a class='btn btn-default btn-xs cancel' href='#'><i class='fa fa-remove'/></a>")

clearErrors = ($row) ->
  $row.find("td[data] span, td[data-name] span").html('')

setErrors = ($row, data) ->
  for k,v of data.responseJSON
    $row.find("td[data='#{k}'] span, td[data-name='#{k}'] span").html(v.join())

dispatchChangeEvent = ($table, type) ->
  window.dispatchEvent new CustomEvent('editable_table_changed', detail: {resource: $table.data('resource'), type: type})

selectFromData = (data, query) ->
  jQuery("<div>#{data}</div>").find(query)